seperator_scenario_name = "Sc_"
column_name_scenario = "Scenario"
column_name_model = "Model"
column_name_id = "ID"
model_name = "TiMBA"

default_plot_settings = {"line_witdh":1.5,
                         "title_font_size":20,
                         "font_size":16,
                         "tick_font_size":14,
                         "legend_font_size":14}

printing_plot_settings = {"line_witdh":7,
                          "title_font_size":32,
                          "font_size":26,
                          "tick_font_size":24,
                          "legend_font_size":18}